#!/usr/local/bin/perl -w

use strict;
use Digest::MD5;

my($species_id, $pfam_result, $seqs) = @ARGV;

if(!defined $seqs)
{
	print STDERR "Usage: pfam2codadb.pl <species id> <pfam result> <fasta seqs>\n";
	exit;
}


open(SEQ, "<$seqs") or die "$!";
my @seqs = <SEQ>;
my $seq = parse_fasta(\@seqs);
close SEQ;

my %md5s = ();

foreach my $id (sort keys %$seq)
{
	my $md5 = Digest::MD5::md5_hex($seq->{$id});

	$md5s{$id} = $md5;
	print "$id\t$md5\n";
}

open(PFAM, "<$pfam_result") or die "$!";

my %doms = ();

while(<PFAM>)
{
	chomp;

	my(@row) = split /\s+/;

	my $id = $row[0];
	my @id = split /\|/, $id;
	$id = $id[1];

	my $pfam = $row[5];
	# Get rid of version number
	$pfam =~ s/\.\d+//;

	$doms{$id}->{$pfam} = 1;
}
close PFAM;

foreach my $id (sort keys %doms)
{
	my $pfam = join " ", sort keys %{$doms{$id}};

	if(!exists $md5s{$id})
	{
		print STDERR "No sequence was found for $id, could not make MD5, skipping\n";
	}
	else
	{
		print "$species_id\t$md5s{$id}\t$id\t$pfam\n";
	}
}

sub parse_fasta
{
	$/ = "\n";

	my $input = shift;

	my %seqs;
	my $header;
	my $seq;

	foreach my $line (@$input)
	{
		chomp $line;

		if ($line =~ /^>(\S+)/ && defined $seq)
		{
			$seqs{$header} = $seq;
			$header = $1;

			my @header = split /\|/, $header;
			$header = $header[1];

			$seq = '';
		}
		elsif ($line =~ /^>(\S+)/)
		{
			$header = $1;

			my @header = split /\|/, $header;
			$header = $header[1];
		}
		else
		{
			$seq .= $line;
		}
	}

	$seqs{$header} = $seq;
	return \%seqs;
}
